@@ -3,6 +3,8 @@ from __future__ import (unicode_literals, division, absolute_import, print_funct
 
 import os
 import re
+import csv
+import sys
 
 from collections import defaultdict
 
@@ -17,9 +19,12 @@ from powerline.bindings.vim import (vim_get_func, getbufvar, vim_getbufoption,
                                     list_tabpage_buffers_segment_info)
 from powerline.theme import requires_segment_info, requires_filesystem_watcher
 from powerline.lib import add_divider_highlight_group
-from powerline.lib.vcs import guess, tree_status
+from powerline.lib.vcs import guess
 from powerline.lib.humanize_bytes import humanize_bytes
 from powerline.lib import wraps_saveargs as wraps
+from powerline.segments.common.vcs import BranchSegment
+from powerline.segments import with_docstring
+from powerline.lib.unicode import string, unicode
 
 try:
 	from __builtin__ import xrange as range
@@ -57,11 +62,6 @@ vim_modes = {
 }
 
 
-eventfuncs = defaultdict(lambda: [])
-bufeventfuncs = defaultdict(lambda: [])
-defined_events = set()
-
-
 # TODO Remove cache when needed
 def window_cached(func):
 	cache = {}
@@ -480,31 +480,34 @@ def modified_buffers(pl, text='+ ', join_str=','):
 
 @requires_filesystem_watcher
 @requires_segment_info
-def branch(pl, segment_info, create_watcher, status_colors=False):
-	'''Return the current working branch.
+class VimBranchSegment(BranchSegment):
+	divider_highlight_group = 'branch:divider'
 
-	:param bool status_colors:
-		determines whether repository status will be used to determine highlighting. Default: False.
+	@staticmethod
+	def get_directory(segment_info):
+		if vim_getbufoption(segment_info, 'buftype'):
+			return None
+		return buffer_name(segment_info)
 
-	Highlight groups used: ``branch_clean``, ``branch_dirty``, ``branch``.
 
-	Divider highlight group used: ``branch:divider``.
-	'''
-	name = buffer_name(segment_info)
-	skip = not (name and (not vim_getbufoption(segment_info, 'buftype')))
-	if not skip:
-		repo = guess(path=name, create_watcher=create_watcher)
-		if repo is not None:
-			branch = repo.branch()
-			scol = ['branch']
-			if status_colors:
-				status = tree_status(repo, pl)
-				scol.insert(0, 'branch_dirty' if status and status.strip() else 'branch_clean')
-			return [{
-				'contents': branch,
-				'highlight_group': scol,
-				'divider_highlight_group': 'branch:divider',
-			}]
+branch = with_docstring(VimBranchSegment(),
+'''Return the current working branch.
+
+:param bool status_colors:
+	Determines whether repository status will be used to determine highlighting. 
+	Default: False.
+:param bool ignore_statuses:
+	List of statuses which will not result in repo being marked as dirty. Most 
+	useful is setting this option to ``["U"]``: this will ignore repository 
+	which has just untracked files (i.e. repository with modified, deleted or 
+	removed files will be marked as dirty, while just untracked files will make 
+	segment show clean repository). Only applicable if ``status_colors`` option 
+	is True.
+
+Highlight groups used: ``branch_clean``, ``branch_dirty``, ``branch``.
+
+Divider highlight group used: ``branch:divider``.
+''')
 
 
 @requires_filesystem_watcher
@@ -622,3 +625,118 @@ def winnr(pl, segment_info, show_current=True):
 	winnr = segment_info['winnr']
 	if show_current or winnr != vim.current.window.number:
 		return str(winnr)
+
+
+csv_cache = None
+sniffer = csv.Sniffer()
+
+
+def detect_text_csv_dialect(text, display_name, header_text=None):
+	return (
+		sniffer.sniff(string(text)),
+		sniffer.has_header(string(header_text or text)) if display_name == 'auto' else display_name,
+	)
+
+
+CSV_SNIFF_LINES = 100
+CSV_PARSE_LINES = 10
+
+
+if sys.version_info < (2, 7):
+	def read_csv(l, dialect, fin=next):
+		try:
+			return fin(csv.reader(l, dialect))
+		except csv.Error as e:
+			if str(e) == 'newline inside string' and dialect.quotechar:
+				# Maybe we are inside an unfinished quoted string. Python-2.6 
+				# does not handle this fine
+				return fin(csv.reader(l[:-1] + [l[-1] + dialect.quotechar]))
+			else:
+				raise
+else:
+	def read_csv(l, dialect, fin=next):
+		return fin(csv.reader(l, dialect))
+
+
+def process_csv_buffer(pl, buffer, line, col, display_name):
+	global csv_cache
+	if csv_cache is None:
+		csv_cache = register_buffer_cache(defaultdict(lambda: (None, None, None)))
+	try:
+		cur_first_line = buffer[0]
+	except UnicodeDecodeError:
+		cur_first_line = vim.eval('strtrans(getline(1))')
+	dialect, has_header, first_line = csv_cache[buffer.number]
+	if dialect is None or (cur_first_line != first_line and display_name == 'auto'):
+		try:
+			text = '\n'.join(buffer[:CSV_SNIFF_LINES])
+		except UnicodeDecodeError:  # May happen in Python 3
+			text = vim.eval('join(map(getline(1, {0}), "strtrans(v:val)"), "\\n")'.format(CSV_SNIFF_LINES))
+		try:
+			dialect, has_header = detect_text_csv_dialect(text, display_name)
+		except csv.Error as e:
+			pl.warn('Failed to detect csv format: {0}', str(e))
+			# Try detecting using three lines only:
+			if line == 1:
+				rng = (0, line + 2)
+			elif line == len(buffer):
+				rng = (line - 3, line)
+			else:
+				rng = (line - 2, line + 1)
+			try:
+				dialect, has_header = detect_text_csv_dialect(
+					'\n'.join(buffer[rng[0]:rng[1]]),
+					display_name,
+					header_text='\n'.join(buffer[:4]),
+				)
+			except csv.Error as e:
+				pl.error('Failed to detect csv format: {0}', str(e))
+				return None, None
+	if len(buffer) > 2:
+		csv_cache[buffer.number] = dialect, has_header, cur_first_line
+	column_number = len(read_csv(
+		buffer[max(0, line - CSV_PARSE_LINES):line - 1] + [buffer[line - 1][:col]],
+		dialect=dialect,
+		fin=list,
+	)[-1]) or 1
+	if has_header:
+		try:
+			header = read_csv(buffer[0:1], dialect=dialect)
+		except UnicodeDecodeError:
+			header = read_csv([vim.eval('strtrans(getline(1))')], dialect=dialect)
+		column_name = header[column_number - 1]
+	else:
+		column_name = None
+	return unicode(column_number), column_name
+
+
+@requires_segment_info
+def csv_col_current(pl, segment_info, display_name='auto', name_format=' ({column_name:.15})'):
+	'''Display CSV column number and column name
+
+	Requires filetype to be set to ``csv``.
+
+	:param bool or str name:
+		May be ``True``, ``False`` and ``"auto"``. In the first case value from 
+		the first raw will always be displayed. In the second case it will never 
+		be displayed. In thi last case ``csv.Sniffer().has_header()`` will be 
+		used to detect whether current file contains header in the first column.
+	:param str name_format:
+		String used to format column name (in case ``display_name`` is set to 
+		``True`` or ``"auto"``). Accepts ``column_name`` keyword argument.
+
+	Highlight groups used: ``csv:column_number`` or ``csv``, ``csv:column_name`` or ``csv``.
+	'''
+	if vim_getbufoption(segment_info, 'filetype') != 'csv':
+		return None
+	line, col = segment_info['window'].cursor
+	column_number, column_name = process_csv_buffer(pl, segment_info['buffer'], line, col, display_name)
+	if not column_number:
+		return None
+	return [{
+		'contents': column_number,
+		'highlight_group': ['csv:column_number', 'csv'],
+	}] + ([{
+		'contents': name_format.format(column_name=column_name),
+		'highlight_group': ['csv:column_name', 'csv'],
+	}] if column_name else [])
